import VuePlugin from 'rollup-plugin-vue';
import commonjs from '@rollup/plugin-commonjs';
import postcss from 'rollup-plugin-postcss';
import babel from '@rollup/plugin-babel';
import path from 'path';
import typescript from 'rollup-plugin-typescript2';
import { terser } from 'rollup-plugin-terser';
import json from '@rollup/plugin-json';
import url from '@rollup/plugin-url';
import del from 'rollup-plugin-delete';
import { join, relative, dirname } from 'path';
import node from '@rollup/plugin-node-resolve';
import tsConfig from './tsconfig.json';
import replace from '@rollup/plugin-replace';
const PKGJSON = require('./package.json');
const OUTPUT_DIR = tsConfig.compilerOptions.outDir;
const ROOT_DIR = tsConfig.compilerOptions.rootDir;
const NORMALIZER_PATH = join(__dirname, 'rollup-utils', 'normalize-component.js').replace(/\\/g, '/');
const STYLE_INJECTOR_PATH = join(__dirname, 'rollup-utils', 'style-injector.js').replace(/\\/g, '/');
// #region input
/**
 * @returns {Object.<string,string>}
 */
function getRollupInput() {
    const input = {};
    function assignToInput(path) {
        if (typeof path === 'string') {
            const key = relative(OUTPUT_DIR, path).replace('.js', '');
            const value = join(ROOT_DIR, relative(OUTPUT_DIR, path).replace('js', 'ts'));
            input[key] = value;
        }
    }
    const panels = PKGJSON.panels;
    if (panels) {
        for (const key in panels) {
            if (Object.hasOwnProperty.call(panels, key)) {
                const element = panels[key];
                const main = element.main;
                if (main) {
                    assignToInput(main);
                }
            }
        }
    }
    const main = PKGJSON.main;
    if (main) {
        assignToInput(main);
    }
    const contributions = PKGJSON.contributions;
    if (contributions) {
        const scene = PKGJSON.contributions.scene;
        if (scene) {
            const script = scene.script;
            if (script) {
                assignToInput(script);
            }
        }
        const builder = PKGJSON.contributions.builder;
        if (builder) {
            assignToInput(builder);
        }
        const preferences = PKGJSON.contributions.preferences;
        if (preferences) {
            const custom = preferences.custom;
            if (custom) {
                assignToInput(custom);
            }
        }
        const project = PKGJSON.contributions.project;
        if (project) {
            const custom = project.custom;
            if (custom) {
                assignToInput(custom);
            }
        }
    }
    return input;
}
// #endregion
const INPUT = getRollupInput();
// #region Plugins setting

const VUEPLUGIN = VuePlugin({
    defaultLang: { script: 'ts' },
    /**
     * @en Inject style
     * @zh 注入样式
     */
    css: true,
    /**
     * @en Injector style to shadow dom
     * @zh 往shadow dom 注入样式
     */
    isWebComponent: true,
    /**
     * @en This script describes which node to inject style into
     * @zh 这个脚本描述了往哪个节点注入样式
     */
    normalizer: `~${NORMALIZER_PATH}`,
    /**
     * @en This script describes how to inject style
     * @zh 这个脚本描述了如何注入样式
     */
    styleInjectorShadow: `~${STYLE_INJECTOR_PATH}`,
});
/**
 * @en Set the value of "import.meta.relativePath" to the relativePath from the directory where "bundle" is located to the root directory
 * @zh 设置 import.meta.relativePath 的值为 bundle 所在目录到根目录的相对路径
 * @returns 
 */
function RelativeRoot() {
    return {
        name: 'relative-root', // this name will show up in warnings and errors
        resolveImportMeta(property, options) {
            if (property === 'relativePath') {
                return `"${relative(dirname(join(OUTPUT_DIR, options.chunkId)), '').replace(/\\/g, '/')}"`;
            }
            return null;
        },
    };
}
/**
 * @en Parse typescript
 * @zh 解析 typescript
 */
const TYPESCRIPT = typescript({
    tsconfig: path.resolve('tsconfig.json'),
});
const NODE = node({});
const COMMONJS = commonjs();
/**
 * @en Parsing styles so that styles can be imported
 * @zh 解析样式文件，可以导入样式
 */
const POSTCSS = postcss({
    inject: false,
    extract: false,
});
/**
 * @en parse json
 * @zh 解析 json
 */
const JSON_Plugin = json();
// ** READ THIS ** before editing `knownAssetTypes`.
//   If you add an asset to `knownAssetTypes`, make sure to also add it
//   to the TypeScript declaration file `src/shims.d.ts`.
const KNOWN_ASSET_TYPES = [
    // images
    'png',
    'jpe?g',
    'gif',
    'svg',
    'ico',
    'webp',
    'avif',

    // media
    'mp4',
    'webm',
    'ogg',
    'mp3',
    'wav',
    'flac',
    'aac',

    // fonts
    'woff2?',
    'eot',
    'ttf',
    'otf',

    // other
    'wasm',
    'webmanifest',
    'pdf',
    'txt',
];

const DEFAULT_ASSETS_REG_EXP = new RegExp(
    '\\.(' + KNOWN_ASSET_TYPES.join('|') + ')(\\?.*)?$'
);
/**
 * A Rollup plugin which imports files as data-URIs or ES Modules.
 * @see https://www.npmjs.com/package/@rollup/plugin-url
 */
const URL = url({
    limit: 0,
    fileName: '[dirname][name][extname]',
    destDir: OUTPUT_DIR,
    publicPath: OUTPUT_DIR + '/',
    sourceDir: path.join(__dirname, ROOT_DIR),
    include: DEFAULT_ASSETS_REG_EXP,
});
/**
 * @en delete output dir once before compilation
 * @zh 编译前清空输出目录
 */
const DEL_ONCE = del({
    'targets': OUTPUT_DIR + '/*',
    runOnce: true,
});
const REPLACE = replace({
    'process.env.NODE_ENV': JSON.stringify('production'),
});
const RELATIVE_ROOT = RelativeRoot();
/**
 * @en ES6 syntax => ES5 syntax
 * @zh 将 ES6 格式转换为 ES5 格式
 */
const BABEL = babel();
// #endregion

/**
 * @type {import('rollup').RollupOptions['plugins']}
 */
const COMMON_PLUGINS = [
    VUEPLUGIN, TYPESCRIPT, COMMONJS, POSTCSS, BABEL, URL, JSON_Plugin, RELATIVE_ROOT, DEL_ONCE, NODE,
];
/**
 *
 * @param {string} id
 * @en Declare external modules
 * @zh 声明外部模块
 */
function external(id) {
}
/**
 * @type {import('rollup').RollupOptions}
 */
const DEBUG_CONFIG = {
    input: INPUT,
    output: {
        format: 'cjs',
        dir: path.resolve(__dirname, OUTPUT_DIR),
    },
    plugins: [...COMMON_PLUGINS],
    external,
};
/**
 * @type {import('rollup').RollupOptions}
 */
const RELEASE_CONFIG = {
    input: INPUT,
    output: {
        format: 'cjs',
        dir: path.join(__dirname, OUTPUT_DIR),
    },
    plugins: [
        ...COMMON_PLUGINS,
        /**
         * @en uglify scripts
         * @zh 代码混淆
         */
        terser(),
        REPLACE,
    ],
    external,
};

export default (commandLineArgs) => {
    if (commandLineArgs.configDebug === true) {
        return DEBUG_CONFIG;
    }
    return RELEASE_CONFIG;
};

